@@ -16,8 +16,8 @@ import argparse
 import sys
 from typing import Any, Dict, List
 
-from fastestimator.cli.cli_util import SaveAction
 from fastestimator.summary.logs import parse_log_dir
+from fastestimator.util.cli_util import SaveAction
 
 
 def logs(args: Dict[str, Any], unknown: List[str]) -> None:
@@ -44,7 +44,7 @@ def logs(args: Dict[str, Any], unknown: List[str]) -> None:
                   args['pretty_names'])
 
 
-def configure_log_parser(subparsers: argparse.PARSER) -> None:
+def configure_log_parser(subparsers: argparse._SubParsersAction) -> None:
     """Add a logging parser to an existing argparser.
 
     Args:
