@@ -115,6 +115,8 @@ def connect(db_path: Optional[str] = None) -> sql.Connection:
     """
     if db_path is None:
         db_path = os.path.join(str(Path.home()), 'fastestimator_data', 'history.db')
+    if db_path != ':memory:':  # This is a reserved keyword to create an in-memory database
+        os.makedirs(os.path.dirname(db_path), exist_ok=True)  # Make sure folders exist before creating disk file
     connection = sql.connect(db_path, detect_types=sql.PARSE_DECLTYPES | sql.PARSE_COLNAMES)
     connection.execute("PRAGMA foreign_keys = 1")  # Enable FK constraints
     connection.row_factory = sql.Row  # Get nice query return objects
